//
// FTS client library
// Copyright (C) 2001 by IRCAM-Centre Georges Pompidou, Paris, France.
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2.1
// of the License, or (at your option) any later version.
// 
// See file COPYING.LIB for further informations on licensing terms.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 


namespace ircam {
namespace fts {
namespace client {

  template <class KeyT, class ValT> class FTSCLIENT_API Hashtable;
  class BinaryProtocolDecoder;
  class BinaryProtocolEncoder;
  class ReceiveThread;

  class FTSCLIENT_API FtsServerConnection {
  public:
    static const int DEFAULT_RECEIVE_BUFFER_SIZE;
    static const int CLIENT_OBJECT_ID;

    FtsServerConnection() throw( FtsClientException);
    virtual ~FtsServerConnection();

    FtsObject *getObject( int id);
    int putObject( int id, FtsObject *obj);

    void writeInt( int v) throw( FtsClientException);
    void writeDouble( double v) throw( FtsClientException);
    void writeSymbol( const char *v) throw( FtsClientException);
    void writeString( const char *v) throw( FtsClientException);
    void writeRawString( const char *v) throw( FtsClientException);
    void writeObject( const FtsObject *v) throw( FtsClientException);
    /* This version is used for object that have predefined IDs */
    void writeObject( int id) throw( FtsClientException);
    void writeArgs( const FtsArgs &v) throw( FtsClientException);

    void endOfMessage() throw( FtsClientException);

    virtual void close() throw (FtsClientException) = 0;
    
    void startThread() throw(FtsClientException);

    void receiveLoop();
  protected:
    virtual int read( unsigned char *b, int len) throw (FtsClientException) = 0;
    virtual void write( const unsigned char *b, int len) throw (FtsClientException) = 0;

  private:
    int getNewObjectID()
    {
      int id = _newObjectID;
      _newObjectID += 2; 
      return id; 
    }

    int _newObjectID;
    Hashtable< int, FtsObject *> *_objectTable;
    BinaryProtocolEncoder *_encoder;
    BinaryProtocolDecoder *_decoder;
    unsigned char *_receiveBuffer;
    ReceiveThread* _receiveThread;
  };

};
};
};
