#
# jMax
# Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
# 
# See file COPYING.LIB for further informations on licensing terms.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 

# debug
from SymbolCache import SymbolCache
from BinaryProtocol import BinaryProtocol
from FtsClientException import FtsClientException
from HashTable import HashTable

qInitial = 1

qInt0 = 10
qInt1 = 11
qInt2 = 12
qInt3 = 13

qFloat0 = 20
qFloat1 = 21
qFloat2 = 22
qFloat3 = 23
qFloat4 = 24
qFloat5 = 25
qFloat6 = 26
qFloat7 = 27

qString = 30

qObject0 = 40
qObject1 = 41
qObject2 = 42
qObject3 = 43

qSymbolIndex0 = 50
qSymbolIndex1 = 51
qSymbolIndex2 = 52
qSymbolIndex3 = 53

qSymbolCache0 = 60
qSymbolCache1 = 61
qSymbolCache2 = 62
qSymbolCache3 = 63
qSymbolCache4 = 64

class BinaryProtocolDecoder:
    """
    This class decode message using FTS cliet-server protocol
    """
    
    def __init__(self, serverConnection):
        self.__serverConnection = serverConnection
        self.__argsCount = 0
        self.__currentState = qInitial
        self.__lval = 0
        self.__buffer = ""
        self.__selector = ""
        self.__args = []
        self.__symbolCache = SymbolCache()

    def getState(self):
        return self.__currentState
    
    def clearAction(self, input):
        self.__lval = 0
    
    def shiftAction(self, input):
        self.__lval = self.__lval << 8 | input
    
    def shiftLongAction(self, input):
        self.__lval = self.__lval << 8 | input
    
    def bufferClearAction(self, input):
        self.__buffer = ""
    
    def clearAllAction(self, input):
        self.__lval = 0
        self.__buffer = ""

    def bufferShiftAction(self, input):
        self.__buffer += chr(input)
    
    def endIntAction(self, input):
        self.__lval = self.__lval << 8 | input
        if self.__argsCount >= 2:
            self.__args.append(self.__lval)
        self.__argsCount += 1
    
    def endFloatAction(self, input):
        self.__lval = self.__lval << 8 | input
        if self.__argsCount >= 2:
            self.__args.append(self.__lval)
        self.__argsCount += 1
    
    def endSymbolIndexAction(self, input):
        self.__lval = self.__lval << 8 | input
        s = self.__symbolCache.get(self.__lval)
        if self.__argsCount == 1:
            self.__selector = s
        else:
            self.__args.append(s)
        self.__argsCount += 1
    
    def endSymbolCacheAction(self, input):
        symbol = self.__buffer
        self.__symbolCache.put(self.__lval, symbol)

        if self.__argsCount == 1:
            self.__selector = symbol
        else:
            self.__args.append(symbol)
        self.__argsCount += 1
    
    def endStringAction(self, input):
        if self.__argsCount >= 2:
            self.__args.append(self.__buffer)
        self.__argsCount += 1
    
    def endObjectAction(self, input):
        self.__lval = self.__lval << 8 | input
        obj = self.__serverConnection.getObject(self.__lval)
        if self.__argsCount == 0:
            self.__target = obj
        else:
            self.__args.append(obj)
        
        self.__argsCount += 1
    
    def endMessageAction(self, input):
        if hasattr(self.__target, self.__selector):
            apply(getattr(self.__target, self.__selector), self.__args)
        else:
            print "ERROR: no method in", self.__target, " for selector:", self.__selector
        
        self.__args = []
        self.__argsCount = 0
    
    def nextState(self, input):
        if self.__currentState == 0:
            if input == BinaryProtocol.END_OF_MESSAGE:
                self.__currentState = qInitial
        elif self.__currentState == qInitial:
            if input == BinaryProtocol.INT:
                self.__currentState = qInt0
                self.clearAction(input)
            elif input == BinaryProtocol.FLOAT:
                self.__currentState = qFloat0
                self.clearAction(input)
            elif input == BinaryProtocol.SYMBOL_INDEX:
                self.__currentState = qSymbolIndex0
                self.clearAction(input)
            elif input == BinaryProtocol.SYMBOL_CACHE:
                self.__currentState = qSymbolCache0
                self.clearAllAction(input)
            elif input == BinaryProtocol.STRING:
                self.__currentState = qString
                self.bufferClearAction(input)
            elif input == BinaryProtocol.OBJECT:
                self.__currentState = qObject0
                self.clearAction(input)
            elif input == BinaryProtocol.END_OF_MESSAGE:
                self.endMessageAction(input)
            else:
                self.__currentState =0
        
        elif self.__currentState == qInt0:
            self.__currentState = qInt1
            self.shiftAction(input)
        elif self.__currentState == qInt1:
            self.__currentState = qInt2
            self.shiftAction(input)
        elif self.__currentState == qInt2:
            self.__currentState = qInt3
            self.shiftAction(input)
        elif self.__currentState == qInt3:
            self.__currentState = qInitial
            self.endIntAction(input)
        elif self.__currentState == qFloat0:
            self.__currentState = qFloat1
            self.shiftAction(input)
        elif self.__currentState == qFloat1:
            self.__currentState  = qFloat2
            self.shiftAction(input)
        elif self.__currentState == qFloat2:
            self.__currentState = qFloat3
            self.shiftAction(input)
        elif self.__currentState == qFloat3:
            self.__currentState = qFloat4
            self.shiftAction(input)
        elif self.__currentState == qFloat4:
            self.__currentState = qFloat5
            self.shiftAction(input)
        elif self.__currentState == qFloat5:
            self.__currentState = qFloat6
            self.shiftAction(input)
        elif self.__currentState == qFloat6:
            self.__currentState = qFloat7
            self.shiftAction(input)
        elif self.__currentState == qFloat7:
            self.__currentState = qInitial
            self.endFloatAction(input)
        elif self.__currentState == qSymbolIndex0:
            self.__currentState = qSymbolIndex1
            self.shiftAction(input)
        elif self.__currentState == qSymbolIndex1:
            self.__currentState = qSymbolIndex2
            self.shiftAction(input)
        elif self.__currentState == qSymbolIndex2:
            self.__currentState = qSymbolIndex3
            self.shiftAction(input)
        elif self.__currentState == qSymbolIndex3:
            self.__currentState = qInitial
            self.endSymbolIndexAction(input)
        elif self.__currentState == qSymbolCache0:
            self.__currentState = qSymbolCache1
            self.shiftAction(input)
        elif self.__currentState == qSymbolCache1:
            self.__currentState = qSymbolCache2
            self.shiftAction(input)
        elif self.__currentState == qSymbolCache2:
            self.__currentState = qSymbolCache3
            self.shiftAction(input)
        elif self.__currentState == qSymbolCache3:
            self.__currentState = qSymbolCache4
            self.shiftAction(input)
        elif self.__currentState == qSymbolCache4:
            if input == 0:
                self.__currentState = qInitial
                self.endSymbolCacheAction(input)
            else:
                self.bufferShiftAction(input)
        elif self.__currentState == qString:
            if input == 0:
                self.__currentState = qInitial
                self.endStringAction(input)
            else:
                self.bufferShiftAction(input)
        elif self.__currentState == qObject0:
            self.__currentState = qObject1
            self.shiftAction(input)
        elif self.__currentState == qObject1:
            self.__currentState = qObject2
            self.shiftAction(input)
        elif self.__currentState == qObject2:
            self.__currentState = qObject3
            self.shiftAction(input)
        elif self.__currentState == qObject3:
            self.__currentState = qInitial
            self.endObjectAction(input)
    
    def decode(self, data):
        """
        Decode message given as argument
        """
        for c in data:
            self.nextState(ord(c))
            if self.__currentState == 0:
                raise FtsClientException

        
        
