//
// jMax
// Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2.1
// of the License, or (at your option) any later version.
// 
// See file COPYING.LIB for further informations on licensing terms.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
//

package ircam.jmax.guiobj;

import java.io.*;
import java.util.*;

import ircam.jmax.*;
import ircam.jmax.fts.*;
import ircam.fts.client.*;

public class FtsDefineObject extends FtsGraphicObject
{
  static
  {
    FtsObject.registerMessageHandler( FtsDefineObject.class, FtsSymbol.get("type"), new FtsMessageHandler(){
	  public void invoke(FtsObject obj, FtsArgs args)
	  {
	    ((FtsDefineObject)obj).setCurrentType( args.getSymbol(0).toString());
	  }
	});
      FtsObject.registerMessageHandler( FtsDefineObject.class, FtsSymbol.get("expression"), new FtsMessageHandler(){
	  public void invoke(FtsObject obj, FtsArgs args)
	  {
	    ((FtsDefineObject)obj).setCurrentExpression( args.getSymbol(0).toString());
	  }
      });
    }

  public FtsDefineObject(FtsServer server, FtsObject parent, int id, String className, FtsAtom[] args, int offset, int length)
  {
    super(server, parent, id, className, args[offset].stringValue);

    ninlets = 0;
    noutlets = 0;
    
    type = "const";
    expression = "";
  }

  public int getNumberOfInlets()
  {
    return 0;
  }
  public int getNumberOfOutlets()
  {
    return 0;
  }
  
  public void requestSetExpression( String expression)
  {
    this.expression = expression;

    args.clear();
    args.addRawString( expression);

    try
      {
	send( FtsSymbol.get("expression"), args);
      }
    catch( IOException e)
      {
	System.err.println("FtsDefineObject: I/O Error sending expression Message!");
	e.printStackTrace(); 
      }
  }
  
  public void requestSetType( String type)
  {
    this.type = type;

    args.clear();
    args.addSymbol( FtsSymbol.get( type));
    
    try
      {
	send( FtsSymbol.get("type"), args);
      }
    catch( IOException e)
      {
	System.err.println("FtsDefineObject: I/O Error sending setType Message!");
	e.printStackTrace(); 
      }

    ((Define)listener).typeChanged( type);
  }
  
  public String getExpression()
  {
    return expression;
  }
  
  public void setCurrentExpression( String expression)
  {  
    this.expression = expression;
    ((Define)listener).expressionChanged( expression);	
  }
  
  public String getType()
  {
    return type;
  }
  
  public void setCurrentType( String type)
  {
    this.type = type;
    ((Define)listener).updateTypeWidth( type);
  }
  
  private String expression = null;
  private String type = null;
}






