//
// jMax
// Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2.1
// of the License, or (at your option) any later version.
// 
// See file COPYING.LIB for further informations on licensing terms.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 

package ircam.jmax.editors.sequence.track;

import java.io.*;
import java.lang.*;
import java.awt.event.*;
import java.awt.*;

import ircam.jmax.*;
import ircam.jmax.fts.*;
import ircam.jmax.toolkit.*;
import ircam.jmax.toolkit.menus.*;

import javax.swing.*;
import javax.swing.table.*;

import ircam.jmax.editors.sequence.*;
import ircam.jmax.editors.sequence.menus.*;

/**
* This implementation builds a SequencePanel to represent the data.
 */
public class TrackWindow extends JFrame implements EditorContainer{
	
  //------------------- fields
  transient TrackPanel trackPanel;
  FtsTrackObject trackData;
	EditMenu editMenu;
  
  public final static int DEFAULT_WIDTH  = 800;
  public final static int DEFAULT_HEIGHT = 553;
  public final static int MAX_HEIGHT     = 800;
  public final static int EMPTY_HEIGHT   = 94;
  /**
		* Constructor with FtsSequenceObject
   */
  public TrackWindow(FtsTrackObject data)
  {
    super();
		
    trackData = data;
		
    TrackEditorFactoryTable.init();
    
    makeTitle();
		
    //... then the SequencePanel
    trackPanel = new TrackPanel( this, data);
				
    getContentPane().add( trackPanel);
    
    addWindowListener(new WindowListener(){
			public void windowOpened(WindowEvent e){}
			public void windowClosed(WindowEvent e){}
			public void windowClosing(WindowEvent e)
		  {
				trackPanel.close(true);
		  }
			public void windowDeiconified(WindowEvent e){}
			public void windowIconified(WindowEvent e){}
			public void windowActivated(WindowEvent e)
		  {
				TrackEditor editor = trackPanel.getTrackEditor();
				SequenceSelection.setCurrent( editor.getSelection());
	    }
			public void windowDeactivated(WindowEvent e){}
		});
		
		addComponentListener( new ComponentAdapter() {
			public void componentResized(ComponentEvent e)
		  {
				Rectangle bounds = TrackWindow.this.getBounds();
				trackData.editorState.setSize(bounds.width, bounds.height);
		  }
			public void componentMoved(ComponentEvent e)
		  {
				Rectangle bounds = TrackWindow.this.getBounds();
				trackData.editorState.setLocation(bounds.x, bounds.y);
		  }
		});
		
		if( !trackData.editorState.haveContent())
			pack();
		
    if(JMaxApplication.getProperty("no_menus") == null)
      makeMenuBar();
		
    validate();
		if( !trackData.editorState.haveContent())
			pack();
  }
	
  private final void makeTitle(){
    setTitle(MaxWindowManager.getWindowManager().makeUniqueWindowTitle("Track"));
    MaxWindowManager.getWindowManager().windowChanged(this);
  } 
  
  public void setName(String name)
  {
    setTitle( MaxWindowManager.getWindowManager().makeUniqueWindowTitle("Track " + name));
    MaxWindowManager.getWindowManager().windowChanged(this);
  }
	
  private final void makeMenuBar()
  {
    JMenuBar mb = new JMenuBar();
    mb.add( new FileMenu());
    editMenu = new EditMenu(this);
    mb.add(editMenu);
    mb.add(new ircam.jmax.toolkit.menus.MaxWindowJMenu("Windows", this));
		
    setJMenuBar(mb);
  }
	
  public EditMenu getEditMenu()
  {
    return editMenu;
  }
	
  // ------ editorContainer interface ---------------
  public Editor getEditor(){
    return trackPanel;
  }
  public Frame getFrame(){
    return this;
  }
  public Point getContainerLocation(){
    return getLocation();
  }
  public Rectangle getViewRectangle(){
    return trackPanel.scrollTracks.getViewport().getViewRect();
  }
}








